
#include "TimeView.h"

#include <StringView.h>
#include <Box.h>
#include <Button.h>
#include <Size.h>
#include <SpaceLayoutItem.h>
#include <Font.h>
#include <Window.h>
#include <String.h>

#include <GroupLayout.h>
#include <GroupLayoutBuilder.h>

#include <math.h>
#include <iostream>

TimeView :: TimeView( BRect const& frame, int32 const& replyCode,
                        BLooper* replyLooper, BHandler* replyHandler )
    : BView(frame, "time_view", B_FOLLOW_NONE, B_PULSE_NEEDED),
        fReplyCode(replyCode), fReplyLooper(replyLooper), fReplyHandler(replyHandler),
        fIsRunning(false), fIsTurnWhite(true), fTimeWhite(0), fTimeBlack(0),
        fPulseRate( 2 ), fPulses( 0 )
{	
	fWhiteClock = new BStringView( "clock_white", "00:00:00" );	
	fBlackClock = new BStringView( "clock_black", "00:00:00" );

	SetLayout( new BGroupLayout( B_HORIZONTAL ) );
	
	AddChild(
	
		BGroupLayoutBuilder( B_VERTICAL, 10 )
			.AddGroup( B_HORIZONTAL, 5 )
				.Add( fWhiteClock )
				.Add( BSpaceLayoutItem::CreateHorizontalStrut(5) )
				.Add( fBlackClock )
			.End()
	);
   
    Init();
}


void
TimeView :: Init( void )
{
	fWhiteClock->SetViewColor( 0,0,0);
	fBlackClock->SetViewColor( 0,0,0);
	
	fWhiteClock->SetHighColor( 200, 200, 0 );
	fBlackClock->SetHighColor( 200, 200, 0 );
	
	fWhiteClock->SetFontSize( 20 );
	fBlackClock->SetFontSize( 20 );
	
	
	fWhiteClock->Invalidate();
    fBlackClock->Invalidate();
}


void
TimeView::Start( void )
{
    fIsRunning = true;
    
    if( fIsTurnWhite == true ){
        fBlackClock->SetHighColor( 200, 200, 200 );

        if ( fTimeWhite > 0 )
            fWhiteClock->SetHighColor( 255, 255, 0 );
        else
            fWhiteClock->SetHighColor( 255, 0, 0 );

        fBlackClock->Invalidate();
        fWhiteClock->Invalidate();
    }
    else{
        fWhiteClock->SetHighColor( 200, 200, 200 );
        
        if ( fTimeBlack > 0 ) 
            fBlackClock->SetHighColor( 255, 255, 0 );
        else
            fBlackClock->SetHighColor( 255, 0, 0 );
        
        fWhiteClock->Invalidate();
        fBlackClock->Invalidate();
    }
    
}


void
TimeView::Stop( void )
{
    fIsRunning = false;
}


void TimeView :: Pulse( void )
{   
	if( fIsRunning == false )
        return;
    
    fPulses++;
    
    if( fPulses < fPulseRate )
        return;
    
    fPulses = 0;
    
    if (fIsTurnWhite == true){
        fTimeWhite--;
        _DisplayTimeWhite();
        
        if( fTimeWhite == 0 )
            _NotifyTime();
    }
    else{
        fTimeBlack--;
        _DisplayTimeBlack();
        
        if( fTimeBlack == 0 )
            _NotifyTime();
    }
    
    Invalidate();
}


void
TimeView::_NotifyTime( void )
{
    BMessage*   msg = new BMessage( fReplyCode );
    
    msg->AddInt32( "time", fIsTurnWhite );
    
    fReplyLooper->PostMessage( msg, fReplyHandler );
    delete msg;
}


void
TimeView::_DisplayTimeWhite( void )
{
    if ( fTimeWhite > 0 )
        fWhiteClock->SetHighColor( 255, 255, 0 );
    else
        fWhiteClock->SetHighColor( 255, 0, 0 );
 
    fWhiteClock->SetText( _TimeToString(fTimeWhite) );
    fWhiteClock->Invalidate();
}


void
TimeView::_DisplayTimeBlack( void )
{
    if ( fTimeBlack > 0 )
        fBlackClock->SetHighColor( 255, 255, 0 );
    else
        fBlackClock->SetHighColor( 255, 0, 0 );

    fBlackClock->SetText( _TimeToString(fTimeBlack) );
    fBlackClock->Invalidate();
}


void
TimeView::Switch( void )
{
    fIsTurnWhite = !fIsTurnWhite;
    
    if( fIsTurnWhite == true ){
        fBlackClock->SetHighColor( 200, 200, 200 );
        fBlackClock->Invalidate();
        
        if ( fTimeWhite > 0 )
            fWhiteClock->SetHighColor( 255, 255, 0 );
        else
            fWhiteClock->SetHighColor( 255, 0, 0 );
        
        fWhiteClock->Invalidate();
        
    }
    else{
        fWhiteClock->SetHighColor( 200, 200, 200 );
        fWhiteClock->Invalidate();
        
        if ( fTimeBlack > 0 ) 
            fBlackClock->SetHighColor( 255, 255, 0 );
        else
            fBlackClock->SetHighColor( 255, 0, 0 );
        
        fBlackClock->Invalidate();
    }
}


void
TimeView::SetTimeWhite( int const& time )
{
    fTimeWhite = time;
    _DisplayTimeWhite();
}


void
TimeView::SetTimeBlack( int const& time )
{
    fTimeBlack = time;
    _DisplayTimeBlack();
}


void
TimeView::SetTime( int const& time )
{
    SetTimeWhite( time );
    SetTimeBlack( time );
}


BString
TimeView::_TimeToString( int time )
{
    BString  str;
    
    str << " ";
    
    if( time < 0 )
    {
        str << "-";
        time= fabs( time );
	}
    
    int sec= time%60;
	time/=60;
	int min=time%60;
	time/=60;
	int hour=time;
   
    if ( fTimeWhite >= 3600 || fTimeBlack >= 3600 ) 
        str << hour << ":";
    
    if( min < 10 )
        str << "0";
    
    str << min << ":";
    
    if( sec < 10 )  
        str << "0";
    
    str << sec << " ";
   
    
    return( str );
}